@@ -1,10 +1,18 @@
 """Asynchronous Python client for WLED."""
 
-from .models import (  # noqa
+from .exceptions import (
+    WLEDConnectionClosedError,
+    WLEDConnectionError,
+    WLEDConnectionTimeoutError,
+    WLEDError,
+    WLEDUpgradeError,
+)
+from .models import (
     Device,
     Effect,
     Info,
     Leds,
+    LightCapability,
     Live,
     Nightlight,
     Palette,
@@ -15,20 +23,14 @@ from .models import (  # noqa
     State,
     Sync,
 )
-from .wled import (
-    WLED,
-    WLEDConnectionClosed,
-    WLEDConnectionError,
-    WLEDConnectionTimeoutError,
-    WLEDError,
-    WLEDUpgradeError,
-)
+from .wled import WLED
 
 __all__ = [
     "Device",
     "Effect",
     "Info",
     "Leds",
+    "LightCapability",
     "Live",
     "Nightlight",
     "Palette",
@@ -39,7 +41,7 @@ __all__ = [
     "State",
     "Sync",
     "WLED",
-    "WLEDConnectionClosed",
+    "WLEDConnectionClosedError",
     "WLEDConnectionError",
     "WLEDConnectionTimeoutError",
     "WLEDError",
