@@ -4,20 +4,19 @@ from __future__ import annotations
 import asyncio
 import json
 import socket
-from collections.abc import Callable
 from contextlib import suppress
 from dataclasses import dataclass
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 import aiohttp
 import async_timeout
-import backoff  # type: ignore
+import backoff
 from awesomeversion import AwesomeVersion, AwesomeVersionException
 from cachetools import TTLCache
 from yarl import URL
 
 from .exceptions import (
-    WLEDConnectionClosed,
+    WLEDConnectionClosedError,
     WLEDConnectionError,
     WLEDConnectionTimeoutError,
     WLEDEmptyResponseError,
@@ -26,6 +25,12 @@ from .exceptions import (
 )
 from .models import Device, Live, Playlist, Preset
 
+if TYPE_CHECKING:
+    from collections.abc import Callable, Sequence
+
+
+VERSION_CACHE: TTLCache[str, str | None] = TTLCache(maxsize=16, ttl=7200)
+
 
 @dataclass
 class WLED:
@@ -40,13 +45,13 @@ class WLED:
     _device: Device | None = None
     _supports_si_request: bool | None = None
     _supports_presets: bool | None = None
-    _version_cache: TTLCache = TTLCache(maxsize=16, ttl=7200)
 
     @property
     def connected(self) -> bool:
         """Return if we are connect to the WebSocket of a WLED device.
 
-        Returns:
+        Returns
+        -------
             True if we are connected to the WebSocket of a WLED device,
             False otherwise.
         """
@@ -55,7 +60,8 @@ class WLED:
     async def connect(self) -> None:
         """Connect to the WebSocket of a WLED device.
 
-        Raises:
+        Raises
+        ------
             WLEDError: The configured WLED device, does not support WebSocket
                 communications.
             WLEDConnectionError: Error occurred while communicating with
@@ -68,9 +74,8 @@ class WLED:
             await self.update()
 
         if not self.session or not self._device or self._device.info.websocket is None:
-            raise WLEDError(
-                "The WLED device at {self.host} does not support WebSockets"
-            )
+            msg = "The WLED device at {self.host} does not support WebSockets"
+            raise WLEDError(msg)
 
         url = URL.build(scheme="ws", host=self.host, port=80, path="/ws")
 
@@ -81,27 +86,31 @@ class WLED:
             aiohttp.ClientConnectionError,
             socket.gaierror,
         ) as exception:
-            raise WLEDConnectionError(
+            msg = (
                 "Error occurred while communicating with WLED device"
                 f" on WebSocket at {self.host}"
-            ) from exception
+            )
+            raise WLEDConnectionError(msg) from exception
 
     async def listen(self, callback: Callable[[Device], None]) -> None:
         """Listen for events on the WLED WebSocket.
 
         Args:
+        ----
             callback: Method to call when a state update is received from
                 the WLED device.
 
         Raises:
+        ------
             WLEDError: Not connected to a WebSocket.
             WLEDConnectionError: An connection error occurred while connected
                 to the WLED device.
-            WLEDConnectionClosed: The WebSocket connection to the remote WLED
+            WLEDConnectionClosedError: The WebSocket connection to the remote WLED
                 has been closed.
         """
         if not self._client or not self.connected or not self._device:
-            raise WLEDError("Not connected to a WLED WebSocket")
+            msg = "Not connected to a WLED WebSocket"
+            raise WLEDError(msg)
 
         while not self._client.closed:
             message = await self._client.receive()
@@ -119,9 +128,8 @@ class WLED:
                 aiohttp.WSMsgType.CLOSED,
                 aiohttp.WSMsgType.CLOSING,
             ):
-                raise WLEDConnectionClosed(
-                    f"Connection to the WLED WebSocket on {self.host} has been closed"
-                )
+                msg = f"Connection to the WLED WebSocket on {self.host} has been closed"
+                raise WLEDConnectionClosedError(msg)
 
     async def disconnect(self) -> None:
         """Disconnect from the WebSocket of a WLED device."""
@@ -135,7 +143,7 @@ class WLED:
         self,
         uri: str = "",
         method: str = "GET",
-        data: dict | None = None,
+        data: dict[str, Any] | None = None,
     ) -> Any:
         """Handle a request to a WLED device.
 
@@ -143,16 +151,19 @@ class WLED:
         the WLED device.
 
         Args:
+        ----
             uri: Request URI, for example `/json/si`.
             method: HTTP method to use for the request.E.g., "GET" or "POST".
             data: Dictionary of data to send to the WLED device.
 
         Returns:
+        -------
             A Python dictionary (JSON decoded) with the response from the
             WLED device.
 
         Raises:
-            WLEDConnectionError: An error occurred while communitcation with
+        ------
+            WLEDConnectionError: An error occurred while communication with
                 the WLED device.
             WLEDConnectionTimeoutError: A timeout occurred while communicating
                 with the WLED device.
@@ -180,61 +191,74 @@ class WLED:
                     json=data,
                     headers=headers,
                 )
+
+            content_type = response.headers.get("Content-Type", "")
+            if response.status // 100 in [4, 5]:
+                contents = await response.read()
+                response.close()
+
+                if content_type == "application/json":
+                    raise WLEDError(  # noqa: TRY301
+                        response.status,
+                        json.loads(contents.decode("utf8")),
+                    )
+                raise WLEDError(  # noqa: TRY301
+                    response.status,
+                    {"message": contents.decode("utf8")},
+                )
+
+            if "application/json" in content_type:
+                response_data = await response.json()
+                if (
+                    method == "POST"
+                    and uri == "/json/state"
+                    and self._device is not None
+                    and data is not None
+                ):
+                    self._device.update_from_dict(data={"state": response_data})
+            else:
+                response_data = await response.text()
+
         except asyncio.TimeoutError as exception:
-            raise WLEDConnectionTimeoutError(
-                f"Timeout occurred while connecting to WLED device at {self.host}"
-            ) from exception
+            msg = f"Timeout occurred while connecting to WLED device at {self.host}"
+            raise WLEDConnectionTimeoutError(msg) from exception
         except (aiohttp.ClientError, socket.gaierror) as exception:
-            raise WLEDConnectionError(
-                f"Error occurred while communicating with WLED device at {self.host}"
-            ) from exception
+            msg = f"Error occurred while communicating with WLED device at {self.host}"
+            raise WLEDConnectionError(msg) from exception
 
-        content_type = response.headers.get("Content-Type", "")
-        if (response.status // 100) in [4, 5]:
-            contents = await response.read()
-            response.close()
-
-            if content_type == "application/json":
-                raise WLEDError(response.status, json.loads(contents.decode("utf8")))
-            raise WLEDError(response.status, {"message": contents.decode("utf8")})
-
-        if "application/json" in content_type:
-            response_data = await response.json()
-            if (
-                method == "POST"
-                and uri == "/json/state"
-                and self._device is not None
-                and data is not None
-            ):
-                self._device.update_from_dict(data={"state": response_data})
-            return response_data
-
-        return await response.text()
+        return response_data
 
     @backoff.on_exception(
-        backoff.expo, WLEDEmptyResponseError, max_tries=3, logger=None
+        backoff.expo,
+        WLEDEmptyResponseError,
+        max_tries=3,
+        logger=None,
     )
-    async def update(self, full_update: bool = False) -> Device:
+    async def update(self, *, full_update: bool = False) -> Device:  # noqa: PLR0912
         """Get all information about the device in a single call.
 
         This method updates all WLED information available with a single API
         call.
 
         Args:
+        ----
             full_update: Force a full update from the WLED Device.
 
         Returns:
+        -------
             WLED Device data.
 
         Raises:
+        ------
             WLEDEmptyResponseError: The WLED device returned an empty response.
         """
         if self._device is None or full_update:
             if not (data := await self.request("/json")):
-                raise WLEDEmptyResponseError(
+                msg = (
                     f"WLED device at {self.host} returned an empty API"
-                    " response on full update"
+                    " response on full update",
                 )
+                raise WLEDEmptyResponseError(msg)
 
             # Try to get presets, introduced in WLED 0.11
             try:
@@ -244,8 +268,9 @@ class WLED:
             except WLEDError:
                 self._supports_presets = False
 
-            versions = await self.get_wled_versions_from_github()
-            data["info"].update(versions)
+            with suppress(WLEDError):
+                versions = await self.get_wled_versions_from_github()
+                data["info"].update(versions)
 
             self._device = Device(data)
 
@@ -253,7 +278,7 @@ class WLED:
             # a single info and state call
             try:
                 self._supports_si_request = self._device.info.version >= AwesomeVersion(
-                    "0.10.0"
+                    "0.10.0",
                 )
             except AwesomeVersionException:
                 # Could be a manual build one? Lets poll for it
@@ -267,40 +292,46 @@ class WLED:
 
         if self._supports_presets:
             if not (presets := await self.request("/presets.json")):
-                raise WLEDEmptyResponseError(
+                msg = (
                     f"WLED device at {self.host} returned an empty API"
-                    " response on presets update"
+                    " response on presets update",
                 )
+                raise WLEDEmptyResponseError(msg)
             self._device.update_from_dict({"presets": presets})
 
         # Handle legacy state and update in separate requests
         if not self._supports_si_request:
             if not (info := await self.request("/json/info")):
-                raise WLEDEmptyResponseError(
+                msg = (
                     f"WLED device at {self.host} returned an empty API"
-                    " response on info update"
+                    " response on info update",
                 )
+                raise WLEDEmptyResponseError(msg)
 
             if not (state := await self.request("/json/state")):
-                raise WLEDEmptyResponseError(
+                msg = (
                     f"WLED device {self.host} returned an empty API"
-                    " response on state update"
+                    " response on state update",
                 )
+                raise WLEDEmptyResponseError(msg)
 
-            versions = await self.get_wled_versions_from_github()
-            info.update(versions)
+            with suppress(WLEDError):
+                versions = await self.get_wled_versions_from_github()
+                info.update(versions)
 
             self._device.update_from_dict({"info": info, "state": state})
             return self._device
 
         if not (state_info := await self.request("/json/si")):
-            raise WLEDEmptyResponseError(
+            msg = (
                 f"WLED device at {self.host} returned an empty API"
-                " response on state & info update"
+                " response on state & info update",
             )
+            raise WLEDEmptyResponseError(msg)
 
-        versions = await self.get_wled_versions_from_github()
-        state_info["info"].update(versions)
+        with suppress(WLEDError):
+            versions = await self.get_wled_versions_from_github()
+            state_info["info"].update(versions)
 
         self._device.update_from_dict(state_info)
 
@@ -312,10 +343,11 @@ class WLED:
         brightness: int | None = None,
         on: bool | None = None,
         transition: int | None = None,
-    ):
+    ) -> None:
         """Change master state of a WLED Light device.
 
         Args:
+        ----
             brightness: The brightness of the light master, between 0 and 255.
             on: A boolean, true to turn the master light on, false otherwise.
             transition: Duration of the crossfade between different
@@ -335,7 +367,8 @@ class WLED:
 
         await self.request("/json/state", method="POST", data=state)
 
-    async def segment(  # pylint: disable=too-many-locals, too-many-branches
+    # pylint: disable=too-many-locals, too-many-branches
+    async def segment(  # noqa: PLR0912
         self,
         segment_id: int,
         *,
@@ -345,6 +378,10 @@ class WLED:
         color_secondary: tuple[int, int, int, int] | tuple[int, int, int] | None = None,
         color_tertiary: tuple[int, int, int, int] | tuple[int, int, int] | None = None,
         effect: int | str | None = None,
+        individual: Sequence[
+            int | Sequence[int] | tuple[int, int, int, int] | tuple[int, int, int]
+        ]
+        | None = None,
         intensity: int | None = None,
         length: int | None = None,
         on: bool | None = None,
@@ -359,6 +396,7 @@ class WLED:
         """Change state of a WLED Light segment.
 
         Args:
+        ----
             segment_id: The ID of the segment to adjust.
             brightness: The brightness of the segment, between 0 and 255.
             clones: Deprecated.
@@ -366,6 +404,7 @@ class WLED:
             color_secondary: The secondary color of this segment.
             color_tertiary: The tertiary color of this segment.
             effect: The effect number (or name) to use on this segment.
+            individual: A list of colors to use for each LED in the segment.
             intensity: The effect intensity to use on this segment.
             length: The length of this segment.
             on: A boolean, true to turn this segment on, false otherwise.
@@ -383,19 +422,22 @@ class WLED:
                 results in a transition of 400ms.
 
         Raises:
+        ------
             WLEDError: Something went wrong setting the segment state.
         """
         if self._device is None:
             await self.update()
 
         if self._device is None:
-            raise WLEDError("Unable to communicate with WLED to get the current state")
+            msg = "Unable to communicate with WLED to get the current state"
+            raise WLEDError(msg)
 
         state = {}
         segment = {
             "bri": brightness,
             "cln": clones,
             "fx": effect,
+            "i": individual,
             "ix": intensity,
             "len": length,
             "on": on,
@@ -461,11 +503,11 @@ class WLED:
             colors.append(color_tertiary)
 
         if colors:
-            segment["col"] = colors  # type: ignore
+            segment["col"] = colors
 
         if segment:
             segment["id"] = segment_id
-            state["seg"] = [segment]  # type: ignore
+            state["seg"] = [segment]  # type: ignore[assignment]
 
         if transition is not None:
             state["tt"] = transition
@@ -476,18 +518,22 @@ class WLED:
         """Set the default transition time for manual control.
 
         Args:
+        ----
             transition: Duration of the default crossfade between different
                 colors/brightness levels. One unit is 100ms, so a value of 4
                 results in a transition of 400ms.
         """
         await self.request(
-            "/json/state", method="POST", data={"transition": transition}
+            "/json/state",
+            method="POST",
+            data={"transition": transition},
         )
 
     async def preset(self, preset: int | str | Preset) -> None:
         """Set a preset on a WLED device.
 
         Args:
+        ----
             preset: The preset to activate on this WLED device.
         """
         # Find preset if it was based on a name
@@ -510,9 +556,9 @@ class WLED:
         """Set a playlist on a WLED device.
 
         Args:
+        ----
             playlist: The playlist to activate on this WLED device.
         """
-
         # Find playlist if it was based on a name
         if self._device and self._device.playlists and isinstance(playlist, str):
             playlist = next(
@@ -533,16 +579,21 @@ class WLED:
         """Set the live override mode on a WLED device.
 
         Args:
+        ----
             live: The live override mode to set on this WLED device.
         """
         await self.request("/json/state", method="POST", data={"lor": live.value})
 
     async def sync(
-        self, *, send: bool | None = None, receive: bool | None = None
+        self,
+        *,
+        send: bool | None = None,
+        receive: bool | None = None,
     ) -> None:
         """Set the sync status of the WLED device.
 
         Args:
+        ----
             send: Send WLED broadcast (UDP sync) packet on state change.
             receive: Receive broadcast packets.
         """
@@ -561,6 +612,7 @@ class WLED:
         """Control the nightlight function of a WLED device.
 
         Args:
+        ----
             duration: Duration of nightlight in minutes.
             fade: If true, the light will gradually dim over the course of the
                 nightlight duration. If false, it will instantly turn to the
@@ -577,20 +629,17 @@ class WLED:
 
         # Filter out not set values
         nightlight = {k: v for k, v in nightlight.items() if v is not None}
-
-        state: dict[str, Any] = {"nl": nightlight}
-        if on:
-            state["on"] = True
-
-        await self.request("/json/state", method="POST", data=state)
+        await self.request("/json/state", method="POST", data={"nl": nightlight})
 
     async def upgrade(self, *, version: str | AwesomeVersion) -> None:
         """Upgrades WLED device to the specified version.
 
         Args:
+        ----
             version: The version to upgrade to.
 
         Raises:
+        ------
             WLEDUpgradeError: If the upgrade has failed.
             WLEDConnectionTimeoutError: When a connection timeout occurs.
             WLEDConnectionError: When a connection error occurs.
@@ -599,16 +648,25 @@ class WLED:
             await self.update()
 
         if self.session is None or self._device is None:
-            raise WLEDUpgradeError("Unexpected upgrade error; No session or device")
+            msg = "Unexpected upgrade error; No session or device"
+            raise WLEDUpgradeError(msg)
 
-        if self._device.info.architecture not in {"esp8266", "esp32"}:
-            raise WLEDUpgradeError("Upgrade is only supported on ESP8266 and ESP32")
+        if self._device.info.architecture not in {
+            "esp01",
+            "esp02",
+            "esp32",
+            "esp8266",
+        }:
+            msg = "Upgrade is only supported on ESP01, ESP02, ESP32 and ESP8266 devices"
+            raise WLEDUpgradeError(msg)
 
         if not self._device.info.version:
-            raise WLEDUpgradeError("Current version is unknown, cannot perform upgrade")
+            msg = "Current version is unknown, cannot perform upgrade"
+            raise WLEDUpgradeError(msg)
 
         if self._device.info.version == version:
-            raise WLEDUpgradeError("Device already running the requested version")
+            msg = "Device already running the requested version"
+            raise WLEDUpgradeError(msg)
 
         # Determine if this is an Ethernet board
         ethernet = ""
@@ -625,41 +683,49 @@ class WLED:
         update_file = (
             f"WLED_{version}_{self._device.info.architecture.upper()}{ethernet}.bin"
         )
-        download_url = f"https://github.com/Aircoookie/WLED/releases/download/v{version}/{update_file}"
+        download_url = (
+            "https://github.com/Aircoookie/WLED/releases/download"
+            f"/v{version}/{update_file}"
+        )
 
         try:
             async with async_timeout.timeout(self.request_timeout * 10):
                 async with self.session.get(
-                    download_url, raise_for_status=True
+                    download_url,
+                    raise_for_status=True,
                 ) as download:
                     form = aiohttp.FormData()
                     form.add_field("file", await download.read(), filename=update_file)
                     await self.session.post(url, data=form)
         except asyncio.TimeoutError as exception:
-            raise WLEDConnectionTimeoutError(
-                "Timeout occurred while fetching WLED version information from GitHub"
-            ) from exception
+            msg = "Timeout occurred while fetching WLED version information from GitHub"
+            raise WLEDConnectionTimeoutError(msg) from exception
         except aiohttp.ClientResponseError as exception:
             if exception.status == 404:
-                raise WLEDUpgradeError(
-                    f"Requested WLED version '{version}' does not exists"
-                ) from exception
-            raise WLEDUpgradeError(
-                f"Could not download requested WLED version '{version}' from {download_url}"
-            ) from exception
+                msg = f"Requested WLED version '{version}' does not exists"
+                raise WLEDUpgradeError(msg) from exception
+            msg = (
+                f"Could not download requested WLED version '{version}'"
+                f" from {download_url}"
+            )
+            raise WLEDUpgradeError(msg) from exception
         except (aiohttp.ClientError, socket.gaierror) as exception:
-            raise WLEDConnectionError(
-                "Timeout occurred while communicating with GitHub for WLED version information"
-            ) from exception
+            msg = (
+                "Timeout occurred while communicating with GitHub"
+                " for WLED version information"
+            )
+            raise WLEDConnectionError(msg) from exception
 
     @backoff.on_exception(backoff.expo, WLEDConnectionError, max_tries=3, logger=None)
     async def get_wled_versions_from_github(self) -> dict[str, str | None]:
         """Fetch WLED version information from GitHub.
 
-        Returns:
+        Returns
+        -------
             A dictionary of WLED versions, with the key being the version type.
 
-        Raises:
+        Raises
+        ------
             WLEDConnectionTimeoutError: Timeout occurred while fetching WLED
                 version information from GitHub.
             WLEDConnectionError: Timeout occurred while communicating with
@@ -669,8 +735,8 @@ class WLED:
         """
         with suppress(KeyError):
             return {
-                "version_latest_stable": self._version_cache["stable"],
-                "version_latest_beta": self._version_cache["beta"],
+                "version_latest_stable": VERSION_CACHE["stable"],
+                "version_latest_beta": VERSION_CACHE["beta"],
             }
 
         if self.session is None:
@@ -679,19 +745,18 @@ class WLED:
         try:
             async with async_timeout.timeout(self.request_timeout):
                 response = await self.session.get(
-                    "https://api.github.com/repos/Aircoookie/WLED/releases"
+                    "https://api.github.com/repos/Aircoookie/WLED/releases",
+                    headers={"Accept": "application/json"},
                 )
         except asyncio.TimeoutError as exception:
-            raise WLEDConnectionTimeoutError(
-                "Timeout occurred while fetching WLED version information from GitHub"
-            ) from exception
+            msg = "Timeout occurred while fetching WLED version information from GitHub"
+            raise WLEDConnectionTimeoutError(msg) from exception
         except (aiohttp.ClientError, socket.gaierror) as exception:
-            raise WLEDConnectionError(
-                "Timeout occurred while communicating with GitHub for WLED version"
-            ) from exception
+            msg = "Timeout occurred while communicating with GitHub for WLED version"
+            raise WLEDConnectionError(msg) from exception
 
         content_type = response.headers.get("Content-Type", "")
-        if (response.status // 100) in [4, 5]:
+        if response.status // 100 in [4, 5]:
             contents = await response.read()
             response.close()
 
@@ -700,24 +765,29 @@ class WLED:
             raise WLEDError(response.status, {"message": contents.decode("utf8")})
 
         if "application/json" not in content_type:
-            raise WLEDError(
-                "Didn't get a JSON response from GitHub while retrieving version information"
-            )
+            msg = "No JSON response from GitHub while retrieving version information"
+            raise WLEDError(msg)
 
         releases = await response.json()
         version_latest = None
         version_latest_beta = None
         for release in releases:
-            if release["prerelease"] is False and version_latest is None:
+            if (
+                release["prerelease"] is False
+                and "b" not in release["tag_name"].lower()
+                and version_latest is None
+            ):
                 version_latest = release["tag_name"].lstrip("vV")
-            if release["prerelease"] is True and version_latest_beta is None:
+            if (
+                release["prerelease"] is True or "b" in release["tag_name"].lower()
+            ) and version_latest_beta is None:
                 version_latest_beta = release["tag_name"].lstrip("vV")
             if version_latest is not None and version_latest_beta is not None:
                 break
 
         # Cache results
-        self._version_cache["stable"] = version_latest
-        self._version_cache["beta"] = version_latest_beta
+        VERSION_CACHE["stable"] = version_latest
+        VERSION_CACHE["beta"] = version_latest_beta
 
         return {
             "version_latest_stable": version_latest,
@@ -737,15 +807,17 @@ class WLED:
     async def __aenter__(self) -> WLED:
         """Async enter.
 
-        Returns:
+        Returns
+        -------
             The WLED object.
         """
         return self
 
-    async def __aexit__(self, *_exc_info) -> None:
+    async def __aexit__(self, *_exc_info: Any) -> None:
         """Async exit.
 
         Args:
+        ----
             _exc_info: Exec type.
         """
         await self.close()
